@@ -1,7 +1,7 @@
 /*
- * $Id: hpuifilter.c 2225 2010-07-22 21:12:36Z heas $
+ * $Id: hpuifilter.c 2997 2014-12-30 19:20:07Z heas $
  *
- * Copyright (c) 1997-2008 by Terrapin Communications, Inc.
+ * Copyright (c) 1997-2015 by Terrapin Communications, Inc.
  * All rights reserved.
  *
  * This code is derived from software contributed to and maintained by
@@ -101,11 +101,16 @@
 # include <stropts.h>
 #endif
 #include <sys/ioctl.h>
+#if HAVE_SYS_RESOURCE_H
+# include <sys/resource.h>
+#endif
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <termios.h>
 #if HAVE_UTIL_H
 # include <util.h>
+#elif HAVE_LIBUTIL_H
+# include <libutil.h>
 #endif
 
 #define	BUFSZ	(LINE_MAX * 2)
@@ -138,13 +143,13 @@ main(int argc, char **argv, char **ev)
 {
     extern char		*optarg;
     extern int		optind;
-    char		ch,
-			hbuf[BUFSZ],		/* hlogin buffer */
+    char		hbuf[BUFSZ],		/* hlogin buffer */
 			ptyname[FILENAME_MAX + 1],
 			tbuf[BUFSZ],		/* telnet/ssh buffer */
 			tbufstr[5] = {ESC, '\x07', '\r', '\n', '\0'};
     int			bytes,			/* bytes read/written */
 			devnull,
+			i,
 			rval = EX_OK,
 			ptym,			/* master pty */
 			ptys;			/* slave pty */
@@ -164,8 +169,8 @@ main(int argc, char **argv, char **ev)
     if (strrchr(progname, '.') != NULL)
 	*(strrchr(progname, '.')) = '\0';
 
-    while ((ch = getopt(argc, argv, "dhvt:")) != -1 )
-	switch (ch) {
+    while ((i = getopt(argc, argv, "dhvt:")) != -1 )
+	switch (i) {
 	case 'd':
 	    debug++;
 	    break;
@@ -367,7 +372,7 @@ main(int argc, char **argv, char **ev)
 		break;
 	    } else if (bytes > 0) {
 		hlen -= bytes;
-		memcpy(hbuf, hbuf + bytes, hlen + 1);
+		memmove(hbuf, hbuf + bytes, hlen + 1);
 		if (hlen < 1)
 		     pfds[2].events &= ~POLLOUT;
 	    }
@@ -423,7 +428,7 @@ main(int argc, char **argv, char **ev)
 		break;
 	    } else if (bytes > 0) {
 		tlen -= bytes;
-		memcpy(tbuf, tbuf + bytes, tlen + 1);
+		memmove(tbuf, tbuf + bytes, tlen + 1);
 		if (tlen < 1)
 		    pfds[1].events &= ~POLLOUT;
 	    }
@@ -534,7 +539,7 @@ int
 filter(char *buf, int len)
 {
     static regmatch_t	pmatch[1];
-#define	N_REG		15		/* number of regexes in reg[][] */
+#define	N_REG		16		/* number of regexes in reg[][] */
 #define	N_CRs		2		/* number of CR replacements */
     static regex_t	preg[N_REG];
     static char		reg[N_REG][50] = {	/* vt100/220 escape codes */
@@ -553,6 +558,7 @@ filter(char *buf, int len)
 				"\x1B\\[\\?25l",		/* vi */
 				"\x1B\\[K",			/* ce */
 				"\x1B\\[7m",			/* mr - ansi */
+				"\x1B\\[6n",			/* u7 - ansi */
 				"\x07",				/* bell */
 
 				/* replace these with CR */
@@ -593,8 +599,8 @@ filter(char *buf, int len)
 	    if (len - pmatch[0].rm_eo <= 0) {
 		buf[pmatch[0].rm_so] = '\0';
 	    } else {
-		memcpy(buf + pmatch[0].rm_so, buf + pmatch[0].rm_eo,
-		       len - pmatch[0].rm_eo + 1);
+		memmove(buf + pmatch[0].rm_so, buf + pmatch[0].rm_eo,
+			len - pmatch[0].rm_eo + 1);
 	    }
 	    len -= pmatch[0].rm_eo - pmatch[0].rm_so;
 	    /* start over with the first regex */
@@ -616,8 +622,8 @@ filter(char *buf, int len)
 	    if (len - pmatch[0].rm_eo == 0) {
 		buf[pmatch[0].rm_so] = '\0';
 	    } else {
-		memcpy(buf + pmatch[0].rm_so, buf + pmatch[0].rm_eo,
-		       len - pmatch[0].rm_eo + 1);
+		memmove(buf + pmatch[0].rm_so, buf + pmatch[0].rm_eo,
+			len - pmatch[0].rm_eo + 1);
 	    }
 	    len -= pmatch[0].rm_eo - pmatch[0].rm_so;
 	    /* start over with the first CR regex */
